@@ -10,8 +10,8 @@
 #
 # $Project: /Convert-Binary-C $
 # $Author: mhx $
-# $Date: 2005/02/21 10:18:41 +0100 $
-# $Revision: 18 $
+# $Date: 2005/05/27 10:07:45 +0200 $
+# $Revision: 20 $
 # $Source: /token/config.pl $
 #
 ################################################################################
@@ -25,6 +25,7 @@
 use Devel::Tokenizer::C;
 
 @options = qw(
+  UnsignedBitfields
   UnsignedChars
   Warnings
   PointerSize
@@ -49,6 +50,7 @@ use Devel::Tokenizer::C;
   HasCPPComments
   HasMacroVAARGS
   OrderMembers
+  Bitfields
 );
 
 @sourcify = qw(
